/** @file gsConfig.h

    @brief Provides preprocessor directives configuration of G+Smo

    This file is part of the G+Smo library. 
    It is autogenerated from gsConfig.h.in during the cmake
    configuration of G+Smo. If you need to make changes edit the
    original file in the sources and NOT THIS FILE.

    This Source Code Form is subject to the terms of the Mozilla Public
    License, v. 2.0. If a copy of the MPL was not distributed with this
    file, You can obtain one at http://mozilla.org/MPL/2.0/.
    
    Author(s): A. Mantzaflaris
*/

#pragma once

/** Define the G+Smo version number. */
#define GISMO_VERSION    "0.8.1alpha"
#define GISMO_MAJOR      0
#define GISMO_MINOR      8
#define GISMO_PATCH      1

/** Define default coefficient type. */
#define real_t           double

/** Define default index type. */
#define index_t          int

/** Define the file data directory. */
#define GISMO_DATA_DIR "/home/amantzaf/gismoAll/trunk/stable/filedata/"

/* Define the shared_ptr to that found on the system. */
/* #undef STD_SHARED_PTR_FOUND */
#define TR1_SHARED_PTR_FOUND
#define TR1_SHARED_PTR_USE_TR1_MEMORY
/* #undef BOOST_SHARED_PTR_FOUND */

/* Enabled Extensions */
/* #undef GISMO_WITH_PSOLID */
/* #undef GISMO_WITH_ONURBS */
/* #undef GISMO_WITH_MPFR */
/* #undef GISMO_WITH_IPOPT */
#define GISMO_WITH_ADIFF

/* #undef GISMO_WITH_SUPERLU */
/* #undef GISMO_WITH_PARDISO */
/* #undef GISMO_WITH_PASTIX */

/* Determine if only header files should be used. */
#define GISMO_BUILD_LIB

/* Determine whether gismo warnings (eg. deprecated) should be emitted. */
/* #undef GISMO_WARNINGS */

/**
 * @name Eigen options - MUST be defined before Eigen is included
 * @{
 */

/* 
   Eigen options -- these MUST be defined before Eigen is included.

   See: 
   http://eigen.tuxfamily.org/dox/TopicPreprocessorDirectives.html
*/

/*  Index type */
/** Eigen options -- these MUST be defined before Eigen is included.\n
   See: http://eigen.tuxfamily.org/dox/TopicPreprocessorDirectives.html
   
   Index type
*/
#define EIGEN_DEFAULT_DENSE_INDEX_TYPE  index_t

/** Eigen options -- these MUST be defined before Eigen is included.\n
   See: http://eigen.tuxfamily.org/dox/TopicPreprocessorDirectives.html
   
   Index type
*/
#define EIGEN_DEFAULT_SPARSE_INDEX_TYPE index_t

/** Eigen options -- these MUST be defined before Eigen is included.\n
   See: http://eigen.tuxfamily.org/dox/TopicPreprocessorDirectives.html
  
 Matrix storage order
*/
#define EIGEN_DEFAULT_TO_COL_MAJOR

/** Eigen options -- these MUST be defined before Eigen is included.\n
  See: http://eigen.tuxfamily.org/dox/TopicPreprocessorDirectives.html
  
  Compile-time static assertions are replaced by runtime assertions;
  this saves compilation time. */
#define EIGEN_NO_STATIC_ASSERT

/** Eigen options -- these MUST be defined before Eigen is included.\n
    See http://eigen.tuxfamily.org/dox/TopicUsingIntelMKL.html */
/* #undef EIGEN_USE_MKL_ALL */

/**@}*/

/* Using Intel MKL. */
//#define EIGEN_USE_MKL_ALL
