
#pragma once


#include <gsAssembler/gsVisitorNeumann.h>

namespace gismo
{


template <class T>
class gsVisitorNeumannMixed : public gsVisitorNeumann<T>
{
public:
typedef gsVisitorNeumann<T> Base;

public:

    gsVisitorNeumannMixed(const gsFunction<T> & neudata, boundary::side s) : 
    Base(neudata,s)
    { }

    void localToGlobal(const gsDofMapper     & mapper,
                       const gsMatrix<T>     & eliminatedDofs,
                       const int               patchIndex,
                       gsSparseMatrix<T>     & sysMatrix,
                       gsMatrix<T>           & rhsMatrix )
    {
        const index_t usz = mapper.freeSize();
        // Local Dofs to global dofs
        mapper.localToGlobal(actives, patchIndex, actives);
        const index_t numActive = actives.rows();

        // Push element contribution to the global load vector
        for (index_t j=0; j!=numActive; ++j)
        {
            // convert local dof index to global dof index
            const unsigned jj = actives(j);
            if (mapper.is_free_index(jj))
                for (index_t s = 0; s!=localRhs.cols(); ++s)
                    rhsMatrix(jj + s*usz,0) += localRhs(j,s);
        }
    }

private:

    // Neumann function
    using Base::neudata_ptr;
    using Base::side;
    
    // Basis values
    using Base::basisData;
    using Base::actives;
    
    // Normal and Neumann values
    using Base::unormal;
    using Base::neuData;
    
    // Local matrix and rhs
    using Base::localMat;
    using Base::localRhs;
};


} // namespace gismo
