/** @file gsRecipeAssemblerBiharmonic.h

    @brief provides an assembler for the biharmonic equation without
    boundary conditions, inherited from the gsRecipeAssembler.

    This file is part of the G+Smo library.

    This Source Code Form is subject to the terms of the Mozilla Public
    License, v. 2.0. If a copy of the MPL was not distributed with this
    file, You can obtain one at http://mozilla.org/MPL/2.0/.

    Author(s): F. Buchegger
*/

#pragma once

#include <gsRecipeAssembler/gsRecipeAssembler.h>
#include <gsRecipeAssembler/gsPhysicalSpace.h>
#include <gsRecipeAssembler/gsPDEOperators.h>

namespace gismo{

class GISMO_EXPORT gsRecipeAssemblerBiharmonic : public gsRecipeAssembler
{
public:
    const gsFunction<real_t>&        m_f;

    gsRecipeAssemblerBiharmonic(const gsMultiPatch<real_t>                &geo,
                                    const gsFunction<real_t>                  &f,
                                    const std::vector<const gsBasis<real_t>*> &bases,
                                    const gsMapper                            &mapper);

    ~gsRecipeAssemblerBiharmonic()
    { freeAll(m_space); }

    gsRecipe<real_t>    getPatchRecipe     (index_t patch);

    gsRecipe<real_t>    getBoundaryRecipe  (patchSide ps)
    { return gsRecipe<real_t>(); }


    virtual void fixBoundaries  ();

};
}

