/** @file gsSpecializedRemappedBasis.h

    @brief Member in the inheritance family of gsRemappedBasis;
    creates the refinement matrix.

    This file is part of the G+Smo library.

    This Source Code Form is subject to the terms of the Mozilla Public
    License, v. 2.0. If a copy of the MPL was not distributed with this
    file, You can obtain one at http://mozilla.org/MPL/2.0/.

    Author(s): D. Mokris
**/

#pragma once

#include <vector>
#include <gsRemappedBasis/gsRemappedBasis.h>

namespace gismo
{

typedef Eigen::Triplet<real_t> triplet;

/** Constructs the refinement matrix using the methods from the
 * inherited classes (thus pure virtual functions). */
class gsSpecializedRemappedBasis : public gsRemappedBasis
{
protected:
    gsFuncInfo         m_info;
    gsMatrix<real_t>   m_boundingBox;
protected:
    void constructMatrix( gsSparseMatrix<real_t> &result );
    virtual std::vector<index_t> getSelected( unsigned lvl ) = 0;
    virtual std::vector<index_t> getInfluencing( unsigned lvl ) = 0;
    // Add default construction.
    virtual gsSparseMatrix<real_t, RowMajor> getTransition( unsigned startLvl,
                                                            unsigned endLvl ) = 0;
private:
    /** Sum of numbers of selected (!) of the preceding bases.*/
    std::vector<unsigned> m_shiftSele;
};

/* Lessons for life:
 * Instantiating a class with pure virtual functions leads to the error message
 * invalid use of incomplete type 'class gismo::gsSpecializedRemappedBasis'.
 *
 * Words 'virtual' and 'static' go only to the declaration, not to the definition.
 *
 * Iterate by size_t through std::vector and by index_t through gsVector.
*/

} // namespace gismo
