/** @file gsCompositeIncrSmoothnessGeom.h

    @brief Provides declaration of Basis abstract interface.

    This file is part of the G+Smo library.

    This Source Code Form is subject to the terms of the Mozilla Public
    License, v. 2.0. If a copy of the MPL was not distributed with this
    file, You can obtain one at http://mozilla.org/MPL/2.0/.

    Author(s): F. Buchegger
*/

#pragma once

#include <gsCore/gsMultiPatch.h>

#include <gsSmoothPatches/gsCompositeGeom.h>
#include <gsSmoothPatches/gsCompositeSingleBasis.h>
#include <gsSmoothPatches/gsCompositeIncrSmoothnessBasis.h>

namespace gismo
{
template<unsigned d,class T>
class gsCompositeIncrSmoothnessGeom : public gsCompositeGeom<d,T>
{

private:
    typedef gsCompositeSingleBasis<d,T> Basis;
    typedef gsCompositeGeom<d,T> Base;
public:
    /// Default empty constructor
    gsCompositeIncrSmoothnessGeom() : Base() { }

    /// Construct B-Spline by basis and coefficient matrix
    gsCompositeIncrSmoothnessGeom( const gsCompositeIncrSmoothnessBasis<d,T> & basis, const gsMatrix<T> & coefs );

    gsCompositeIncrSmoothnessGeom( gsMultiPatch<T> const & mp,int incrSmoothness = -1,int minEVDistance = -1 );

    gsCompositeIncrSmoothnessGeom( gsMultiPatch<T> const & mp,std::vector<patchCorner> C0List,int incrSmoothness = -1,int minEVDistance = -1 );

    ~gsCompositeIncrSmoothnessGeom() {   } //destructor

    using Base::m_compBasis;

    void setCornerC0(patchCorner const & pc);

    void smoothCornerEdge(const patchCorner& pc,const patchSide& ps,bool updateBasis=true);

    void smoothEverything();

    virtual void uniformRefine(int numKnots=1, int mul=1);

    void refine(const unsigned patch, const gsMatrix<T> &boxes);

    void refineElements(const unsigned patch, std::vector<unsigned> const & boxes);

    void refineElements(std::vector<std::vector<unsigned> > const & boxes);

    void uniformRefineAndSmooth(int numKnots=1);

    void refineAndSmooth(const unsigned patch, const gsMatrix<T> &boxes);

    void refineElementsAndSmooth(const unsigned patch, std::vector<unsigned> const & boxes);

}; // class gsCompositeGeom

} // namespace gismo
