#pragma once

#include <gsHSplines/gsHTensorBasis.h>
#include <gsHSplines/gsTHBSplineBasis.h>

#include <gsSmoothPatches/gsCompositeIncrSmoothnessBasis.h>
#include <gsSmoothPatches/gsCompositeHBasis.h>
#include <gsSmoothPatches/gsCompositeBSplineBasis.h>

#include <gsIO/gsFileData.h>
#include <gsIO/gsReadFile.h>

namespace gismo
{

// to remove
bool readInMultipatch(std::string path,gsMultiPatch<> * & mp)
{
    std::cout << path << "\n" ;
    mp = gsReadFile<>(path);
    if(mp==NULL)
    {
        std::cout << "File could not be read." << "\n" ;
        return false;
    }
    //mp->computeTopology();
    return true;
}

// to remove
bool readInTHB(std::string path,gsHTensorBasis<2> * & hbs)
{
    std::cout << path << "\n" ;
    gsFileData<> data( path );
    hbs = data.getFirst< gsTHBSplineBasis<2> >();
    if(hbs==NULL)
    {
        std::cout << "File could not be read." << "\n" ;
        return false;
    }
    return true;
}

template<unsigned d>
gsCompositeIncrSmoothnessBasis<d,real_t> * getCompBasisFromMultiPatch(const gsMultiPatch<> & mp,int incrSmoothness = -1,int minEVDistance = -1 )
{
    gsCompositeIncrSmoothnessBasis<d,real_t> * compBasis=NULL;
    bool tensorBSpline= true;
    bool hTensor = true;
    std::vector<gsTensorBSplineBasis<d,real_t>* >tensorBases;
    for(unsigned i = 0;i<mp.nPatches();++i)
    {
        tensorBases.push_back(dynamic_cast<gsTensorBSplineBasis<d,real_t> * >(& mp.basis(i)));
        tensorBSpline = tensorBSpline && tensorBases[i]!=NULL;
    }
    if(tensorBSpline)
        compBasis = (new gsCompositeBSplineBasis<d,real_t>(tensorBases,mp,incrSmoothness,minEVDistance));
    else
    {
        std::vector<gsHTensorBasis<d,real_t>* >hBases;
        for(unsigned i = 0;i<mp.nPatches();++i)
        {
            hBases.push_back(dynamic_cast<gsHTensorBasis<d,real_t> * >(& mp.basis(i)));
            hTensor = hTensor && hBases[i]!=NULL;
        }
        if(hTensor)
            compBasis = (new gsCompositeHBasis<d,real_t>(hBases,mp,incrSmoothness,minEVDistance));
    }
    GISMO_ASSERT(tensorBSpline||hTensor,"No suitable basis for gsCompositeBasis found.");
    return compBasis;
}

template<unsigned d>
gsCompositeIncrSmoothnessBasis<d,real_t> * getCompBasisFromMultiPatch_withCoefs(const gsMultiPatch<> & mp, std::vector<gsMatrix<real_t>* >&coefs,int incrSmoothness = -1,int minEVDistance = -1 )
{
    gsCompositeIncrSmoothnessBasis<d,real_t> * compBasis=NULL;
    bool tensorBSpline= true;
    bool hTensor = true;
    std::vector<gsTensorBSplineBasis<d,real_t>* >tensorBases;
    for(unsigned i = 0;i<mp.nPatches();++i)
    {
        tensorBases.push_back(dynamic_cast<gsTensorBSplineBasis<d,real_t> * >(& mp.basis(i)));
        tensorBSpline = tensorBSpline && tensorBases[i]!=NULL;
    }
    if(tensorBSpline)
        compBasis = (new gsCompositeBSplineBasis<d,real_t>(tensorBases,mp,coefs,incrSmoothness,minEVDistance));
    else
    {
        std::vector<gsHTensorBasis<d,real_t>* >hBases;
        for(unsigned i = 0;i<mp.nPatches();++i)
        {
            hBases.push_back(dynamic_cast<gsHTensorBasis<d,real_t> * >(& mp.basis(i)));
            hTensor = hTensor && hBases[i]!=NULL;
        }
        if(hTensor)
            compBasis = (new gsCompositeHBasis<d,real_t>(hBases,mp,coefs,incrSmoothness,minEVDistance));
    }
    GISMO_ASSERT(tensorBSpline||hTensor,"No suitable basis for gsCompositeBasis found.");
    return compBasis;
}

} // end namespace gismo
